;****************************************************************************
; BASIC FUNCTION WHATCPU identifies the CPU
; Modified from COM program in PC MAGAZINE, Vol 14, No 2, 1/24/95, page 241
; M. Larson, 01/12/95
;
; BASIC Syntax:
;   DECLARE FUNCTION WhatCPU%()
;   CPU = WhatCPU
;   PRINT "Your CPU is a"; STR$(CPU)
;
;****************************************************************************

CPUID           MACRO
                db      0Fh,0A2h                ;Macro for CPUID instruction
ENDM
                .Model Medium, Basic
                .code

WhatCPU         proc    far

; Check for an 8086 or 8088 by attempting to clear bits 12-15 of the
; FLAGS register. On an 8086 or 8088, these bits are always set.

                mov     dx, 088                 ;Assume 8086/88
                cli                             ;Interrupts off
                pushf                           ;Save FLAGS
                pushf                           ;Push FLAGS on stack
                pop     ax                      ;Pop FLAGS into AX
                and     ax,0FFFh                ;Clear bits 12-15
                or      ax,5000h                ;Set bits 12 and 14
                push    ax                      ;Push AX on stack
                popf                            ;Pop AX into FLAGS
                pushf                           ;Push FLAGS on stack
                pop     ax                      ;Pop FLAGS into AX
                popf                            ;Restore FLAGS
                sti                             ;Interrupts on
                and     ax,0F000h               ;Clear bits 0-11
                cmp     ax,0F000h               ;Are bits 12-15 set?
                jne     check_286               ;No, then it's a 286 or higher
                jmp     done                    ;Yes, then it's an 8086/88

; Check for a 286 by seeing if bits 12-15 are clear after the previous
; operation. On a 286 running in real mode, these bits are always clear.

check_286:      mov     dx, 286                 ;Assume 286
                or      ax,ax                   ;Are bits 12-15 clear?
                jne     check_386               ;No, then it's a 386 or higher
                jmp     done                    ;Yes, then it's a 286

; Check for a 386 by attempting to toggle the EFLAGS register's Alignment
; Check (AC) bit. This bit cannot be changed on a 386.

                .386

check_386:      mov     dx, 386                 ;Assume 386
                cli                             ;Interrupts off
                pushfd                          ;Save EFLAGS
                pushfd                          ;Push EFLAGS on stack
                pop     eax                     ;Pop EFLAGS into EAX
                mov     ebx,eax                 ;Store EAX in EBX
                xor     eax,40000h              ;Toggle bit 18
                push    eax                     ;Push EAX on stack
                popfd                           ;Pop EAX into EFLAGS
                pushfd                          ;Push EFLAGS on stack
                pop     eax                     ;Pop EFLAGS into EAX
                popfd                           ;Restore EFLAGS
                sti                             ;Interrupts on
                and     eax,40000h              ;Clear all but bit 18
                and     ebx,40000h              ;in EAX and EBX
                cmp     eax,ebx                 ;Compare EAX and EBX
                je      done                    ;If equal, then it's a 386

; Check for a 486 by attempting to toggle the EFLAGS register's ID bit.
; This bit cannot be changed on *most* 486s.

                mov     dx, 486                 ;Assume 486
                cli                             ;Interrupts off
                pushfd                          ;Save EFLAGS
                pushfd                          ;Push EFLAGS on stack
                pop     eax                     ;Pop EFLAGS into EAX
                mov     ebx,eax                 ;Store EAX in EBX
                xor     eax,200000h             ;Toggle bit 21
                push    eax                     ;Push EAX on stack
                popfd                           ;Pop EAX into EFLAGS
                pushfd                          ;Push EFLAGS on stack
                pop     eax                     ;Pop EFLAGS into EAX
                popfd                           ;Restore EFLAGS
                sti                             ;Interrupts on
                and     eax,200000h             ;Clear all but bit 21 in
                and     ebx,200000h             ;EAX and EBX
                cmp     eax,ebx                 ;Compare EAX and EBX
                je      done                    ;If equal, then it's a 486

; We succeeded in toggling the processor's ID bit, so it's probably a
; Pentium or higher. But it *could* be a 486, so be careful. To identify
; the processor, use the family ID code returned by the CPUID instruction.

                mov     eax,1                   ;Get CPU ID information
                CPUID                           ;via the CPUID instruction
                and     eax,0F00h               ;Clear all but bits 8-11
                shr     eax,8                   ;Shift right 8 places
                                                ;(EAX = Family ID)

                mov     dx, 586                 ;Assume Pentium
                cmp     eax,5                   ;Is the family ID 5?
                je      done                    ;Yes, then it's a Pentium

                mov     dx, 100                 ;It's not a Pentium,
                imul    dl                      ;so report the family
                add     ax, 86                  ;ID directly
                mov     dx, ax

; Return the processor type in AX

done:           mov    ax, dx
                ret
WhatCPU         endp
                end
